/* wmudmount
 * Copyright © 2010  Brad Jorsch <anomie@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"

#include "die.h"
#include "asklabel.h"

// Custom handler to limit length by bytes rather than characters
static void insert_text(GtkEditable *editable, gchar *new_text, gint new_text_length, gint *position, gpointer user_data){
    struct udisks_fs_info *fs = (struct udisks_fs_info *)user_data;
    int len = strlen(gtk_entry_get_text((GtkEntry*)editable));
    if(new_text_length < 0) new_text_length = strlen(new_text);
    if(len+new_text_length > fs->max_label_len){
        // New text is too long. What we want to do is insert only as much of
        // the new text as will fit, but the only way to do so is to stop this
        // insertion completely and send a new insertion for the reduced text.
        g_signal_stop_emission_by_name(editable, "insert-text");
        gdk_display_beep(gtk_widget_get_display(GTK_WIDGET(editable)));

        gchar *p=new_text;
        while(p<new_text+new_text_length){
            gchar *n = g_utf8_find_next_char(p, new_text+new_text_length);
            if(!n || len+(n-new_text) > fs->max_label_len) break;
            p = n;
        }
        new_text_length = p - new_text;
        if(new_text_length) gtk_editable_insert_text(editable, new_text, new_text_length, position);
    }
}

char *ask_for_label(udisks_volume *v, GtkWidget *transient_for){
    GtkWidget *dialog = gtk_dialog_new_with_buttons("Relabel device",
                             GTK_WINDOW(transient_for),
                             GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
                             "_Relabel", GTK_RESPONSE_ACCEPT,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             NULL);
    gtk_dialog_set_default_response((GtkDialog*)dialog, GTK_RESPONSE_ACCEPT);

    GtkBox *content = (GtkBox*)gtk_dialog_get_content_area((GtkDialog*)dialog);

    GtkBox *hbox = (GtkBox*)gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start(content, (GtkWidget*)hbox, FALSE, FALSE, 10);
    GtkWidget *image = gtk_image_new_from_gicon(v->icon, GTK_ICON_SIZE_DIALOG);
    gtk_misc_set_alignment((GtkMisc*)image, 0.5, 0.0);
    gtk_box_pack_start(hbox, image, FALSE, FALSE, 0);
    GtkWidget *label = gtk_label_new(NULL);
    char *txt = g_markup_printf_escaped("<b><big>%s</big></b>", v->desc);
    gtk_label_set_markup((GtkLabel*)label, txt);
    g_free(txt);
    gtk_misc_set_alignment((GtkMisc*)label, 0.0, 0.5);
    gtk_label_set_line_wrap((GtkLabel*)label, TRUE);
    gtk_box_pack_start(hbox, label, FALSE, FALSE, 0);

    txt = g_strdup_printf("To relabel %s, enter the label here. The maximum label length is %d bytes.", v->desc, v->d.fs.info->max_label_len);
    label = gtk_label_new(txt);
    g_free(txt);
    gtk_misc_set_alignment((GtkMisc*)label, 0.0, 0.5);
    gtk_label_set_line_wrap((GtkLabel*)label, TRUE);
    gtk_box_pack_start(content, label, FALSE, FALSE, 0);

    hbox = (GtkBox*)gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start(content, (GtkWidget*)hbox, FALSE, FALSE, 0);
    label = gtk_label_new_with_mnemonic("_Label:");
    gtk_misc_set_alignment((GtkMisc*)label, 1.0, 0.5);
    gtk_box_pack_start(hbox, label, FALSE, FALSE, 0);
    GtkWidget *entry = gtk_entry_new();
    if(v->label){
        gtk_entry_set_text((GtkEntry*)entry, v->label);
        gtk_editable_select_region((GtkEditable*)entry, 0, -1);
    }
    g_signal_connect(entry, "insert-text", G_CALLBACK(insert_text), (gpointer)v->d.fs.info);
    gtk_entry_set_activates_default((GtkEntry*)entry, TRUE);
    gtk_box_pack_start(hbox, entry, TRUE, TRUE, 0);
    gtk_label_set_mnemonic_widget((GtkLabel*)label, entry);

    gtk_widget_show_all((GtkWidget*)content);
    int response = gtk_dialog_run((GtkDialog*)dialog);
    char *newlabel = NULL;
    switch(response){
      case GTK_RESPONSE_ACCEPT:
        newlabel = g_strdup(gtk_entry_get_text((GtkEntry*)entry));
        break;
    }
    gtk_widget_destroy(dialog);
    return newlabel;
}
